package net.appice.appcurso.publico.controller;

import java.security.MessageDigest;
import java.util.Date;

import net.appice.appcurso.EmailHelper;
import net.appice.appcurso.pedagogica.dao.MatriculaDAO;
import net.appice.appcurso.pessoas.model.Email;
import net.appice.appcurso.publico.bean.Login;
import net.appice.appcurso.publico.dao.NextAuthorizationDAO;
import net.appice.appcurso.publico.service.Base64;
import net.appice.appcurso.publico.service.PasswordHash;
import net.appice.appcurso.seguranca.dao.UsuarioDAO;
import net.appice.appcurso.seguranca.dao.UsuarioPapelDAO;
import net.appice.appcurso.seguranca.model.Papel;
import net.appice.appcurso.seguranca.model.Usuario;

import org.nextframework.authorization.AuthenticationController;
import org.nextframework.controller.Controller;
import org.nextframework.controller.DefaultAction;
import org.nextframework.controller.MultiActionController;
import org.nextframework.core.standard.MessageType;
import org.nextframework.core.web.NextWeb;
import org.nextframework.core.web.WebRequestContext;
import org.springframework.web.servlet.ModelAndView;

@Controller(path = "/publico/login")
public class LoginController extends org.nextframework.authorization.LoginController {

	private PasswordHash passwordHash;

	private UsuarioDAO usuarioDAO;
	private MatriculaDAO matriculaDAO;
	private UsuarioPapelDAO usuarioPapelDAO;

	private static final String AFTER_LOGIN_GO_TO = "/dashboard/index";

	NextAuthorizationDAO appiceAuthorizationDAO;

	/**
	 * Action que envia para a pgina de login
	 * 
	 * @throws Exception
	 */
	@DefaultAction
	public ModelAndView doPage(WebRequestContext request, Login login)
			throws Exception {
		return new ModelAndView("login", "login", login);
	}

	public ModelAndView doForgotPassword(WebRequestContext request, Email email)
			throws Exception {
		return new ModelAndView("forgotPassword", "email", email);
	}

	public ModelAndView doSolicitar(WebRequestContext request, Email email)
			throws Exception {

		Usuario usuario = usuarioDAO.isUsuarioBy(email.getCpf(),
				email.getEndereco());

		if (usuario != null) {
			EmailHelper emailHelper = new EmailHelper(
					"App SETAD - Alterao de senha");
			emailHelper.addDestinario(email.getEndereco());
			String serverName = NextWeb.getApplicationContext()
					.getServletContext().getInitParameter("APP_CONTEXT_PATH");
			emailHelper
					.addPragraf("Recebemos uma solicitao para alterar a senha, basta acessar o link a seguir. Caso no tenha solicitado a alterao, favor desconsiderar essa mensagem.");
			emailHelper.addPragraf("Senha: " + usuario.getLembreteSenha());
			emailHelper.sendMail();
			return new ModelAndView("solicitacaoEfetuada");
		}

		return new ModelAndView("solicitacaoNaoEfetuada");
	}

	/**
	 * Efetua o login do usurio
	 * 
	 * @throws Exception
	 */
	public ModelAndView doLogin(WebRequestContext request, Login bean)
			throws Exception {
		String login = bean.getUsuario() != null ? bean.getUsuario().getValue()
				: null;
		String senha = bean.getSenha();

		// se foi passado o login na requisio, iremos verificar se o usurio
		// existe e a senha est correta
		if (login != null && !"".equals(login) && senha != null
				&& !"".equals(senha)) {
			// buscamos o usurio do banco pelo login
			Usuario userByLogin = appiceAuthorizationDAO.findUserByLogin(login);

			// Validao se o usurio est habilitado
			if (userByLogin != null && userByLogin.getHabilitado()) {

				if (isDataValidadeSenha(userByLogin.getValidadeSenha())) {

					// se o usurio existe e a senha est correta
					if (userByLogin != null
							&& userByLogin.getPassword().equals(
									passwordHash.generateSaltedHash(senha,
											login))) {

						// Setando o atributo de seo USER fazemos o login
						// do
						// usurio
						// no sistema.
						request.setUserAttribute("USER", userByLogin);

						// TODO trocar cliente por CFC
						// Setando o ID do usuario no escopo da aplicao
						// para
						// validar se o usurio ja est conectado.
						// USUARIOS.add(userByLogin.getLogin());
						// request.setAttribute("usuarios", USUARIOS);
						// Setar o cliente do usurio.
						// if (userByLogin.getCliente() != null
						// && userByLogin.getCliente().getId() != null) {
						// request.setUserAttribute("CLIENTE",
						// userByLogin.getCliente());
						// }

						// Limpamos o cache de permisses o menu.
						// O menu ser refeito levando em considerao as
						// permisses
						// do
						// usurio
						// request.setUserAttribute(MenuTag.MENU_CACHE_MAP,
						// null);

						Papel papel = usuarioPapelDAO
								.carregaPapelBy(userByLogin);

						// TODO Colocar tela exclusiva para CFC e para
						// Administrador
						if (papel.getName().equals("Aluno")
								&& !matriculaDAO.isDataLimiteBy(userByLogin
										.getPessoa())) {

							return new ModelAndView(
									"redirect:/pedagogica/realizacaoCurso");
						} else if (papel.getName() != "Aluno") {
							return new ModelAndView("redirect:"
									+ AFTER_LOGIN_GO_TO);
						}

						request.addMessage(
								"J se esgotaram todos os prazos para usufruir dos produtos contratados. ",
								MessageType.ERROR);

						// TODO return com = Caso queira se matricular
						// novamente,
						// clique no link a seguir (link que direciona para a
						// pgina de matrcula - CDU-
						// 0485). Neste caso, vincular a matrcula ao CFC com
						// pes_id =
						// tb_parametro.cfc_id_appice.

					} else {
						// Se o login e/ou a senha no estiverem corretos,
						// avisar o
						// usurio
						request.addMessage("Login e/ou senha invlidos",
								MessageType.ERROR);
					}
				} else {
					request.addMessage(
							"Usurio com a data validade senha expirada!",
							MessageType.ERROR);
					return new ModelAndView("redirect:/publico/alterarSenha");
				}
			} else {
				request.addMessage(
						"Login e/ou senha invlidos ou usurio no habilitado!",
						MessageType.ERROR);
				bean.setUsuario(null);
			}
		} else {
			request.addMessage("Login e/ou senha no informado.",
					MessageType.ERROR);
		}

		// limpar o campo senha, e enviar para a tela de login j que o processo
		// falhou
		bean.setSenha(null);
		return doPage(request, bean);
	}

	public Boolean isDataValidadeSenha(Date dataValidaSenha) {
		if (dataValidaSenha == null)
			return true;
		return dataValidaSenha.after(new Date());
	}

	public String generateSaltedHash(String password, String saltPhrase,
			String algorithm) {
		try {
			MessageDigest md = MessageDigest.getInstance(algorithm);

			if (saltPhrase != null) {
				md.update(saltPhrase.getBytes());
				byte[] salt = md.digest();

				md.reset();
				md.update(password.getBytes());
				md.update(salt);
			} else {
				md.update(password.getBytes());
			}

			byte[] raw = md.digest();
			return Base64.encodeBytes(raw);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public void setPasswordHash(PasswordHash passwordHash) {
		this.passwordHash = passwordHash;
	}

	public void setAppiceAuthorizationDAO(
			NextAuthorizationDAO appiceAuthorizationDAO) {
		this.appiceAuthorizationDAO = appiceAuthorizationDAO;
	}

	public void setUsuarioPapelDAO(UsuarioPapelDAO usuarioPapelDAO) {
		this.usuarioPapelDAO = usuarioPapelDAO;
	}

	public void setUsuarioDAO(UsuarioDAO usuarioDAO) {
		this.usuarioDAO = usuarioDAO;
	}

	public void setMatriculaDAO(MatriculaDAO matriculaDAO) {
		this.matriculaDAO = matriculaDAO;
	}

}
